#include "../Tests.hpp"
#include "core/ArrayList.hpp"
#include "core/Test.hpp"

template class Core::ArrayList<size_t, 20>;
using IntList = Core::ArrayList<size_t, 20>;

static void testAdd() {
    IntList list;
    list.add(5u);
    TEST(5, list[0]);
    TEST(5, list.getLast());
    const IntList& cList = list;
    TEST(5, cList.getLast());
    TEST(1, list.getLength());
}

static void testMultipleAdd() {
    IntList list;
    list.add(4u).add(3u).add(2u);
    TEST(4, list[0]);
    TEST(3, list[1]);
    TEST(2, list[2]);
    TEST(3, list.getLength());
}

static void testAddReplace() {
    IntList list;
    list.add(5u);
    list[0] = 3;
    TEST(3, list[0]);
}

static void testClear() {
    IntList list;
    list.add(5u).add(4u);
    list.clear();
    TEST(0, list.getLength());
}

static void testOverflow(bool light) {
    IntList list;
    for(size_t i = 0; i < 20; i++) {
        list.add(i);
    }
    size_t limit = light ? 1000 : 100'000;
    for(size_t i = 0; i < limit; i++) {
        list.add(i);
    }
    for(size_t i = 0; i < list.getLength(); i++) {
        TEST(i, list[i]);
    }
}

static void testCopy() {
    IntList list;
    list.add(1u).add(2u).add(3u);
    IntList copy(list);
    TEST(list.getLength(), copy.getLength());
    for(size_t i = 0; i < copy.getLength() && i < list.getLength(); i++) {
        TEST(list[i], copy[i]);
    }
}

static void testCopyAssignment() {
    IntList list;
    list.add(1u).add(2u).add(3u);
    IntList copy;
    copy = list;
    TEST(list.getLength(), copy.getLength());
    for(size_t i = 0; i < copy.getLength() && i < list.getLength(); i++) {
        TEST(list[i], copy[i]);
    }
}

static void testMove() {
    IntList list;
    list.add(1u).add(2u).add(3u);
    IntList move(Core::move(list));
    TEST(0, list.getLength());
    TEST(3, move.getLength());
    TEST(1, move[0]);
    TEST(2, move[1]);
    TEST(3, move[2]);
}

static void testMoveAssignment() {
    IntList list;
    list.add(1u).add(2u).add(3u);
    IntList move;
    move = Core::move(list);
    TEST(0, list.getLength());
    TEST(3, move.getLength());
    TEST(1, move[0]);
    TEST(2, move[1]);
    TEST(3, move[2]);
}

static void testToString() {
    IntList list;
    list.add(1u).add(243u).add(423u);
    TEST_STRING("[1, 243, 423]", list);
    TEST_STRING("[1]", IntList().add(1u));
    TEST_STRING("[]", IntList());
}

static void testRemoveBySwap() {
    IntList list;
    list.add(4u).add(3u).add(2u);
    list.removeBySwap(0);
    TEST(2, list[0]);
    TEST(3, list[1]);
    TEST(2, list.getLength());
    list.removeBySwap(1);
    TEST(2, list[0]);
    TEST(1, list.getLength());
    list.removeBySwap(0);
    TEST(0, list.getLength());
}

static void testRemove() {
    IntList list;
    list.add(4u).add(3u).add(2u);
    list.remove(0);
    TEST(3, list[0]);
    TEST(2, list[1]);
    TEST(2, list.getLength());
    list.remove(1);
    TEST(3, list[0]);
    TEST(1, list.getLength());
    list.removeLast();
    TEST(0, list.getLength());
}

static void testForRange() {
    IntList list;
    list.add(1u).add(2u).add(3u);
    for(size_t& i : list) {
        i++;
    }
    for(size_t i = 0; i < list.getLength(); i++) {
        TEST(i + 2, list[i]);
    }
}

void testArrayList(bool light) {
    testAdd();
    testMultipleAdd();
    testAddReplace();
    testClear();
    testOverflow(light);
    testCopy();
    testCopyAssignment();
    testMove();
    testMoveAssignment();
    testToString();
    testRemoveBySwap();
    testRemove();
    testForRange();
}
